@@ -12,7 +12,7 @@ from sklearn.preprocessing import StandardScaler
 
 from . import q, ut, pd, sp, np, warnings, sc
 from .analysis import _compute_csim
-from .utils import prepend_var_prefix, to_vn, substr, sparse_knn
+from .utils import prepend_var_prefix, to_vn, substr, sparse_knn, df_to_dict
 
 from numba.core.errors import NumbaPerformanceWarning, NumbaWarning
 
@@ -58,7 +58,7 @@ class SAMAP(object):
             Path to the `maps` directory output by `map_genes.sh`.
             By default assumes it is in the local directory.
 
-        names1 & names2 : list of 2D tuples or Nx2 numpy.ndarray, optional, default None
+        names : dict of list of 2D tuples or Nx2 numpy.ndarray, optional, default None
             If BLAST was run on a transcriptome with Fasta headers that do not match
             the gene symbols used in the dataset, you can pass a list of tuples mapping
             the Fasta header name to the Dataset gene symbol:
@@ -67,8 +67,9 @@ class SAMAP(object):
             By default, the Fasta header IDs are assumed to be equivalent to the
             gene symbols used in the dataset.
 
-            names1 corresponds to the mapping for organism 1 and names2 corresponds to
-            the mapping for organism 2.
+            The above mapping should be contained in a dicitonary keyed by the corresponding species.
+            For example, if we have `hu` and `mo` species and the `hu` BLAST results need to be translated,
+            then `names = {'hu' : mapping}, where `mapping = [(Fasta header 1, Gene symbol 1), ... , (Fasta header n, Gene symbol n)]`.
 
         gnnm : tuple(scipy.sparse.csr_matrix,numpy array, numpy array)
             If the homology graph was already computed, you can pass it here in the form of a tuple:
@@ -161,8 +162,8 @@ class SAMAP(object):
                 ids, f_maps=f_maps, reciprocate=True
             )            
             if names is not None:
-                gnnm, gns, gns_dict  = _coarsen_blast_graph(
-                    gnnm, gns_list, gns, ids, names
+                gnnm, gns_dict, gns  = _coarsen_blast_graph(
+                    gnnm, gns, names
                 )
 
             gnnm = _filter_gnnm(gnnm, thr=0.25)
@@ -314,16 +315,35 @@ class SAMAP(object):
             print("Running UMAP on the stitched manifolds.")
             sc.tl.umap(self.samap.adata,min_dist=0.1,init_pos='random')
         
+        
+        ix = pd.Series(data = np.arange(samap.adata.shape[1]),index = samap.adata.var_names)[gns].values
+        rixer = pd.Series(index =np.arange(gns.size), data = ix)
+        
         try:
             hom_graph = smap.GNNMS_corr[-1]
-            samap.adata.uns["homology_graph_reweighted"] = hom_graph
+            x,y = hom_graph.nonzero()
+            d = hom_graph.data
+            hom_graph = sp.sparse.coo_matrix((d,(rixer[x].values,rixer[y].values)),shape=(samap.adata.shape[1],)*2).tocsr()                    
+            samap.adata.varp["homology_graph_reweighted"] = hom_graph
+            self.gnnm_refined = hom_graph            
         except:
             pass
         
-        samap.adata.uns["homology_graph"] = gnnm
-        samap.adata.uns["homology_gene_names"] = gns
+        x,y = gnnm.nonzero()
+        d = gnnm.data
+        gnnm = sp.sparse.coo_matrix((d,(rixer[x].values,rixer[y].values)),shape=(samap.adata.shape[1],)*2).tocsr()                            
+        samap.adata.varp["homology_graph"] = gnnm
         samap.adata.uns["homology_gene_names_dict"] = gns_dict
-
+        
+        
+        self.gnnm = gnnm
+        self.gns = q(samap.adata.var_names)
+        
+        gns_dict = {}
+        for sid in ids:
+            gns_dict[sid] = self.gns[np.in1d(self.gns,q(self.sams[sid].adata.var_names))]
+        self.gns_dict = gns_dict
+        
         if umap:
             for sid in ids:
                 sams[sid].adata.obsm['X_umap_samap'] = self.samap.adata[sams[sid].adata.obs_names].obsm['X_umap']     
@@ -471,6 +491,78 @@ class SAMAP(object):
         
         return ax    
     
+    def query_gene_pairs(self,gene):
+        """ Get BLAST and correlation scores of all genes connected
+        to the query gene.
+
+        Preferrably, genes are prepended with their species IDs.
+        For example, "hu_SOX2" instead of "SOX2".
+        
+        Returns: Dictionary with "blast" and "correlation" keys with
+        the BLAST and correlation scores respectively for the queried
+        gene.
+        """ 
+
+        ids = self.ids
+        qgene = None
+        if (gene in self.gns):
+            qgene = gene
+        else:
+            for sid in ids:
+                if sid+'_'+gene in self.gns:
+                    qgene = sid+'_'+gene
+                    break
+        if qgene is None:
+            raise ValueError(f"Query gene {gene} not found in dataset.")
+
+        a = self.gnnm[self.gns==qgene]
+        b = self.gnnm_refined[self.gns==qgene]
+
+        i1 = self.gns[a.nonzero()[1]]
+        i2 = self.gns[b.nonzero()[1]]
+        d1 = a.data
+        d2 = b.data
+        a = pd.Series(index=i1,data=d1)
+        b = pd.Series(index=i2,data=d2)
+        return {"blast":a,"correlation":b}    
+
+    def query_gene_pair(self,gene1,gene2):
+        """ Get BLAST and correlation score for a pair of genes.
+        
+        Preferrably, genes are prepended with their species IDs.
+        For example, "hu_SOX2" instead of "SOX2".
+        
+        Returns: Dictionary with "blast" and "correlation" keys with
+        the BLAST and correlation scores respectively for the queried
+        gene pair.
+        """
+        ids = self.ids
+        qgene1 = None
+        if (gene1 in self.gns):
+            qgene1 = gene1
+        else:
+            for sid in ids:
+                if sid+'_'+gene1 in self.gns:
+                    qgene1 = sid+'_'+gene1
+                    break
+        if qgene1 is None:
+            raise ValueError(f"Query gene {gene1} not found in dataset.")
+
+        qgene2 = None
+        if (gene2 in self.gns):
+            qgene2 = gene2
+        else:
+            for sid in ids:
+                if sid+'_'+gene2 in self.gns:
+                    qgene2 = sid+'_'+gene2
+                    break
+        if qgene2 is None:
+            raise ValueError(f"Query gene {gene2} not found in dataset.")
+
+        a = self.gnnm[self.gns==qgene1].A.flatten()[self.gns==qgene2][0]
+        b = self.gnnm_refined[self.gns==qgene1].A.flatten()[self.gns==qgene2][0]
+        return {"blast":a,"correlation":b}  
+
     def scatter(self,axes=None,COLORS=None,ss=None,**kwargs):  
         
         if ss is None:
@@ -1062,93 +1154,68 @@ def _calculate_blast_graph(ids, f_maps="maps/", eval_thr=1e-6, reciprocate=False
     
     return gnnm, gns, gns_dict
 
-def _coarsen_blast_graph(gnnm, gns_dict, gns, ids, names):
-    # TODO: Update this for multi-species
-    if namesA is not None:
-        groupsA = {}
-        for i in range(len(namesA)):
-            name = namesA[i][1]
-            L = groupsA.get(name, [])
-            L.append(namesA[i][0])
-            groupsA[name] = L
-    else:
-        groupsA = None
+def _coarsen_blast_graph(gnnm, gns, names):
+    sps = np.array([x.split('_')[0] for x in gns])
+    sids = np.unique(sps)
+    ss=[]
+    for sid in sids:
+        n = names.get(sid,None)
+        if n is not None:
+            n = np.array(n)
+            n = (sid+'_'+n.astype('object')).astype('str')
+            s1 = pd.Series(index=n[:,0],data=n[:,1])
+            g = gns[sps==sid]
+            g = g[np.in1d(g,n[:,0],invert=True)]
+            s2 = pd.Series(index=g,data = g)
+            s = pd.concat([s1,s2])
+        else:
+            s = pd.Series(index=gns[sps==sid],data = gns[sps==sid])
+        ss.append(s)
+    ss = pd.concat(ss)
 
-    if namesB is not None:
-        groupsB = {}
-        for i in range(len(namesB)):
-            name = namesB[i][1]
-            L = groupsB.get(name, [])
-            L.append(namesB[i][0])
-            groupsB[name] = L
-    else:
-        groupsB = None
+    x,y = gnnm.nonzero() #get nonzeros
+    s = pd.Series(data=gns,index=np.arange(gns.size)) # convert indices to gene pairs
+    xn,yn = s[x].values,s[y].values 
+    xg,yg = ss[xn].values,ss[yn].values #convert gene pairs to translated
 
-    for n, groups, ids, g in zip(
-        ["A", "B"], [groupsA, groupsB], [id1, id2], [gn1, gn2]
-    ):
-        if groups is not None:
-            if isinstance(groups, dict):
-                xdim = len(list(groups.keys()))
-                for I in range(2):
-                    print(
-                        "Coarsening gene connectivity graph using labels `{}`, round {}.".format(
-                            ids, I
-                        )
-                    )
-                    X = []
-                    Y = []
-                    D = []
-                    for i, k in enumerate(groups.keys()):
-                        if len(groups[k]) > 1:
-                            f = np.in1d(gn, [ids + "_" + x for x in groups[k]])
-                            if f.sum() > 0:
-                                z = gnnm[f].max(0).A.flatten()
-                            else:
-                                z = np.array([])
-                        else:
-                            f = gn == ids + "_" + groups[k][0]
-                            if np.any(f):
-                                z = gnnm[f].A.flatten()
-                            else:
-                                z = np.array([])
-                        y = z.nonzero()[0]
-                        d = z[y]
-                        x = np.ones(y.size) * i
-                        X.extend(x)
-                        Y.extend(y)
-                        D.extend(d)
-                    if n == "A":
-                        xa, ya = gnnm[gn1.size :].nonzero()
-                        da = gnnm[gn1.size :].data
-                        xa += xdim
-                    else:
-                        xa, ya = gnnm[: gn1.size].nonzero()
-                        da = gnnm[: gn1.size].data
-                        X = list(np.array(X) + gn1.size)
-                    X.extend(xa)
-                    Y.extend(ya)
-                    D.extend(da)
-                    gnnm = sp.sparse.coo_matrix(
-                        (D, (X, Y)),
-                        shape=(xdim + gnnm.shape[0] - g.size, gnnm.shape[1]),
-                    ).T.tocsr()
-                g = np.array([ids + "_" + x for x in list(groups.keys())])
-                if n == "A":
-                    gn1 = g
-                else:
-                    gn2 = g
-                gn = np.append(gn1, gn2)
-            else:
-                raise TypeError(
-                    "Gene groupings ({}) must be in dictionary form.".format(n)
-                )
-    f = gnnm.sum(1).A.flatten() != 0
+    da=gnnm.data
+
+    zgu,ix,ivx,cu = np.unique(np.array([xg,yg]).astype('str'),axis=1,return_counts=True,return_index=True,return_inverse=True) # find unique pairs
+
+    xgu,ygu = zgu[:,cu>1] # extract pairs that appear duplicated times
+    xgyg=xg.astype('object')+';'+yg.astype('object')
+    xguygu=xgu.astype('object')+';'+ygu.astype('object')
+
+    filt = np.in1d(xgyg,xguygu)
+
+    DF=pd.DataFrame(data=xgyg[filt][:,None],columns=['key'])
+    DF['val']=da[filt]
+
+    dic = df_to_dict(DF,key_key='key')
+
+    xgu = q([x.split(';')[0] for x in dic.keys()])
+    ygu = q([x.split(';')[1] for x in dic.keys()])
+    replz = q([max(dic[x]) for x in dic.keys()])
+
+    xgu1,ygu1 = zgu[:,cu==1] # get non-duplicate pairs
+    xg = np.append(xgu1, xgu) # append duplicate pairs
+    yg = np.append(ygu1, ygu)
+    da = np.append(da[ix][cu==1],replz) # append duplicate scores to the non-duplicate scores
+    gn = np.unique(np.append(xg,yg)) # get the unique genes
+
+    s = pd.Series(data=np.arange(gn.size),index=gn) # create an indexer
+    xn,yn = s[xg].values,s[yg].values # convert gene pairs to indexes
+    gnnm = sp.sparse.coo_matrix((da,(xn,yn)),shape=(gn.size,)*2).tocsr() # create sparse matrix
+
+    f = gnnm.sum(1).A.flatten() != 0 #eliminate zero rows/columns
     gn = gn[f]
-    gn1 = gn1[np.in1d(gn1, gn)]
-    gn2 = gn2[np.in1d(gn2, gn)]
-    gnnm = gnnm[f, :][:, f]
-    return gnnm, gn1, gn2, gn
+    sps = np.array([x.split('_')[0] for x in gn])
+
+    gns_dict={}
+    for sid in sids:
+        gns_dict[sid] = gn[sps==sid]
+
+    return gnnm, gns_dict, gn
 
 
 def prepare_SAMap_loadings(sam, npcs=300):
